package go.jugadores;
import java.util.*;
import go.Ficha;
import go.Cruce;
import go.Partida;
import go.Cerco;

/**
 * @author  emi
 */
abstract public class Jugador implements Runnable{
	public static final int H = 1;
	public static final int M = 2;
	private int color;
	/**
	 * @uml.property  name="tipo"
	 */
	private int tipo;
	private ArrayList<Ficha> movimientos;
	private ArrayList<Cerco> cercos;
	private int capturadas;
	/**
	 * @uml.property  name="jugada"
	 * @uml.associationEnd  
	 */
	private Ficha jugada;
	/**
	 * @uml.property  name="partida"
	 * @uml.associationEnd  
	 */
	private Partida partida;
	/**
	 * @uml.property  name="pasar"
	 */
	private boolean pasar;
	private transient Thread hilo;
	/**
	 * @uml.property  name="parar"
	 */
	private boolean parar;
	public Jugador(int color, int tipo){
		this.movimientos=new ArrayList<Ficha>();
		this.cercos=new ArrayList<Cerco>();
		this.tipo=tipo;
		this.color=color;
		this.capturadas=0;
		this.parar=false;
		this.pasar=false;
	}
	public int getColorJug(){
		return this.color;
	}
	public String color(){
		if(this.color==1)return "blanco";
		else return "negro";
	}
	/**
	 * @return
	 * @uml.property  name="tipo"
	 */
	public int getTipo(){
		return this.tipo;
	}
	/**
	 * @return
	 * @uml.property  name="jugada"
	 */
	public Ficha getJugada(){
		return this.jugada;
	}
	/**
	 * @param ficha
	 * @uml.property  name="jugada"
	 */
	public void setJugada(Ficha ficha){
		this.jugada=ficha;
	}
	/**
	 * @return
	 * @uml.property  name="partida"
	 */
	public Partida getPartida(){
		return this.partida;
	}
	/**
	 * @param partida
	 * @uml.property  name="partida"
	 */
	public void setPartida(Partida partida){
		this.partida=partida;
	}
	/**
	 * @param parar
	 * @uml.property  name="parar"
	 */
	public void setParar(boolean parar){
		this.parar=parar;
	}
	/**
	 * @return
	 * @uml.property  name="parar"
	 */
	public boolean getParar(){
		return this.parar;
	}
	public boolean esParable(){
		return false;
	}
	/**
	 * @return
	 * @uml.property  name="pasar"
	 */
	public boolean getPasar(){
		return this.pasar;
	}
	/**
	 * @param pasar
	 * @uml.property  name="pasar"
	 */
	public void setPasar(boolean pasar){
		this.pasar=pasar;
	}
	public void parar() {
		//si sabemos que el jugador va a colaborar, pararlo
		//"por las buenas"
		if(esParable()){
			setParar(true);
			try{
				this.hilo.join();
			}catch(InterruptedException ie){}   	  	 
		}
		//si no, pararlo "por las malas"
		else this.hilo.stop();
	}
	public void run() {
		setJugada(calcularMejorJugada(partida));
	}
	public ArrayList<Ficha> getListaMovimientos(){
		return this.movimientos;
	}
	public void addMovimiento(Ficha f){
		this.movimientos.add(f);
	}
	public ArrayList<Cerco> getListaCercos(){
		return this.cercos;
	}
	public void addCerco(Cerco cerco){
		this.cercos.add(cerco);
	}
	public int getNumeroCapturas(){
		return this.capturadas;
	}
	
	//HECHO POR MI
	public void setCapturadas(int a){
		this.capturadas=a;
	}
	
	
	public void sumaCapturas(int capturadas){
		this.capturadas=this.getNumeroCapturas()+capturadas;
	}
	public void hacerJugada(Partida partida){
		this.jugada = null;
		this.hilo = new Thread(this);
		this.hilo.setName("Jugador " + Partida.nomColores[color-1]);
		this.partida = partida;
		this.hilo.start();
	}
	public String toString(){
		return "Jugador de color: "+color+" y de tipo:"+tipo;
	}
	abstract public Ficha calcularMejorJugada(Partida p);
	abstract public String getNombre();
}
